"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDocPerOp = createDocPerOp;
var definitionRefs;

function createDocPerOp(doc) {
  if (doc.definitions.length === 1) {
    var name = doc.definitions[0].name;
    return name ? {
      default: doc,
      [name.value]: doc
    } : {
      default: doc
    };
  }

  definitionRefs = {};
  doc.definitions.forEach(function (def) {
    if (def.name) {
      var refs = new Set();
      collectRefs(def, refs);
      definitionRefs[def.name.value] = refs;
    }
  });
  var docs = {};
  doc.definitions.forEach(function (op, i) {
    if (op.kind === 'OperationDefinition') {
      if (!op.name) {
        throw new Error('Names are required for a document with multiple Queries/Mutations');
      }

      var curOpDoc = createSingleOperationDoc(doc, op.name.value);

      if (i === 0) {
        docs.default = curOpDoc;
        docs[op.name.value] = curOpDoc;
      } else {
        docs[op.name.value] = curOpDoc;
      }
    }
  });
  return docs;
}

function collectRefs(_ref, refs) {
  var kind = _ref.kind,
      type = _ref.type,
      name = _ref.name,
      selectionSet = _ref.selectionSet,
      variableDefinitions = _ref.variableDefinitions,
      definitions = _ref.definitions;

  if (kind === 'FragmentSpread') {
    refs.add(name.value);
  } else if (kind === 'VariableDefinition') {
    var t = type;
    if (t.kind === 'NamedType') refs.add(type.name.value);
    if (t.kind === 'NonNullType' && t.type.kind === 'NamedType') refs.add(type.type.name.value);
  } // Call recursively for types that may contain FragmentSpread or NamedType, if those types exist


  definitions && definitions.forEach(function (def) {
    return collectRefs(def, refs);
  });
  selectionSet && selectionSet.selections.forEach(function (sel) {
    return collectRefs(sel, refs);
  });
  variableDefinitions && variableDefinitions.forEach(function (def) {
    return collectRefs(def, refs);
  });
}

function findOperation(doc, name) {
  return doc.definitions.find(function (op) {
    return op.name ? op.name.value === name : false;
  });
}

function createSingleOperationDoc(doc, operationName) {
  // Copy the DocumentNode, but clear out the definitions
  var newDoc = Object.assign({}, doc);
  newDoc.definitions = [findOperation(doc, operationName)]; // For the current operation, find any fragments referenced by: it, or fragments it references

  var allRefs = new Set();
  var newRefs = new Set(definitionRefs[operationName]);

  while (newRefs.size > 0) {
    var prevRefs = newRefs;
    newRefs = new Set();
    prevRefs.forEach(function (refName) {
      if (!allRefs.has(refName)) {
        allRefs.add(refName);
        var childRefs = definitionRefs[refName] || new Set();
        childRefs.forEach(function (childRef) {
          return newRefs.add(childRef);
        });
      }
    });
  }

  allRefs.forEach(function (refName) {
    var op = findOperation(doc, refName);
    if (op) newDoc.definitions.push(op);
  });
  return newDoc;
}